/*
 * Copyright (C) 2019-2022  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "src/keyset/keyset_info_reply.hh"

#include "src/util/struct_to_string.hh"

namespace Fred {
namespace Registry {
namespace Keyset {

std::string KeysetInfoReply::Data::to_string() const
{
    return Util::StructToString().add("keyset_handle", keyset_handle)
                                 .add("keyset_id", keyset_id)
                                 .add("keyset_history_id", keyset_history_id)
                                 .add("dns_keys", dns_keys)
                                 .add("technical_contacts", technical_contacts)
                                 .add("auth_info", auth_info)
                                 .add("representative_events", representative_events)
                                 .add("sponsoring_registrar", sponsoring_registrar)
                                 .finish();
}

KeysetInfoReply::Exception::KeysetDoesNotExist::KeysetDoesNotExist(const KeysetInfoRequest& src)
    : Registry::Exception("keyset " + src.to_string() + " does not exist")
{
}

std::string KeysetIdReply::Data::to_string() const
{
    return Util::StructToString().add("keyset_id", keyset_id)
                                 .add("keyset_history_id", keyset_history_id)
                                 .finish();
}

KeysetIdReply::Exception::KeysetDoesNotExist::KeysetDoesNotExist(const KeysetIdRequest& src)
    : Registry::Exception("keyset " + src.to_string() + " does not exist")
{
}

} // namespace Fred::Registry::Keyset
} // namespace Fred::Registry
} // namespace Fred
