/*
 * Copyright (C) 2022  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "src/util/into.hh"

#include <boost/test/unit_test.hpp>

#include <type_traits>

static_assert(std::is_same<std::chrono::system_clock::duration, std::chrono::nanoseconds>::value);

namespace {

auto make_time_point(std::chrono::nanoseconds fraction)
{
    static const auto t_sec = std::chrono::system_clock::from_time_t(1652451931); // 2022-05-13 14:25:31
    return t_sec + fraction;
}

}//namespace {anonymous}

BOOST_AUTO_TEST_SUITE(TestUtil)

BOOST_AUTO_TEST_CASE(time_point_nsec_into_string)
{
    const auto t = make_time_point(std::chrono::nanoseconds{123'456'789});
    const auto str_time_point = Fred::Registry::Util::Into<std::string>::from(t);
    BOOST_CHECK_EQUAL(str_time_point, "2022-05-13 14:25:31.123456789");
}

BOOST_AUTO_TEST_CASE(time_point_usec_cropped_into_string)
{
    const auto t = make_time_point(std::chrono::nanoseconds{123'456'789});
    const auto t_usec = std::chrono::time_point_cast<std::chrono::microseconds>(t);
    const auto str_time_point = Fred::Registry::Util::Into<std::string>::from(t_usec);
    BOOST_CHECK_EQUAL(str_time_point, "2022-05-13 14:25:31.123456");
}

BOOST_AUTO_TEST_CASE(time_point_usec_zero_into_string)
{
    const auto t = make_time_point(std::chrono::nanoseconds{123'000'000});
    const auto t_usec = std::chrono::time_point_cast<std::chrono::microseconds>(t);
    const auto str_time_point = Fred::Registry::Util::Into<std::string>::from(t_usec);
    BOOST_CHECK_EQUAL(str_time_point, "2022-05-13 14:25:31.123000");
}

BOOST_AUTO_TEST_SUITE_END()//TestUtil
